/* @(#)fitsmdb.c	14.1.1.1 (ESO-DMD) 09/16/99 09:21:26 */
/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
  MA 02139, USA.
 
  Correspondence concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
===========================================================================*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT  (c)  1996  European Southern Observatory
.IDENT      fitsmdb.c
.LAUGUAGE   C
.AUTHOR     P.Grosbol   ESO/IPG
.KEYWORDS   MIDAS descriptor buffer
.COMMENT    Define buffer for MIDAS descriptors
.VERSION    1.0  1991-Mar-19 : Creation,   PJG
.VERSION    1.1  1993-Feb-23 : Return pointer to structure,   PJG
.VERSION    1.2  1993-Jul-05 : Save keyword commends,   PJG
.VERSION    1.3  1993-Sep-27 : strlen() used for SCDWRC() instead of 72. CG.
.VERSION    1.4  1993-Oct-26 : Update to new SC + prototypes, PJG
.VERSION    1.5  1993-Dec-13 : Disable SC-error handling for descriptors, PJG
.VERSION    1.6  1994-Jan-11 : Check zero length desc. history, PJG
.VERSION    1.7  1996-Oct-22 : Correct length of descriptor names, PJG
---------------------------------------------------------------------*/
#include   <fitsfmt.h>
#include   <fitsdef.h>
#include   <midas_def.h>

static     int          nmdb;    /* no. of entries in keyword buffer */
static     MDBUF  mdb[MXMDB];    /* MIDAS descriptor buffer          */

MDBUF *mdb_init()
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE    initiate the MIDAS descriptor buffer
.RETURN     pointer to internal MIDAS descriptor buffer 
---------------------------------------------------------------------*/
{
  int  i;

  nmdb = 0;
  for (i=0; i<MXMDB; i++) {
    mdb[i].type = '\0';
    mdb[i].val.i = 0;
    mdb[i].idx = 0;
    mdb[i].desc[0] = '\0';
    mdb[i].pcom = (char *) 0;
    mdb[i].buf[0] = '\0';
  }

  return mdb;
}

mdb_put(kw,kwd)
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE    put keyword values into MIDAS descriptor buffer
.RETURN     Status 0:OK, 1:error, keyword not stored
---------------------------------------------------------------------*/
KWORD   *kw;
KWDEF   *kwd;
{
  int  i;
  char *pc, *ps;

  if (nmdb<MXMDB) {               /* save keyword in buffer if space */
    ps = mdb[nmdb].desc;
    pc = kwd->desc;
    for (i=0; i<MXMDN; i++) *ps++ = *pc++;
    mdb[nmdb].type = kwd->type;
    mdb[nmdb].idx = kwd->idx;
    ps = mdb[nmdb].buf;
    switch (kwd->type) {
       case 'S' : mdb[nmdb].val.pc = mdb[nmdb].buf;
                  pc = kw->val.pc;
                  while (*ps++ = *pc++);
                  break;
       case 'I' : mdb[nmdb].val.i = kw->val.i; break;
       case 'R' :
       case 'D' : mdb[nmdb].val.d[0] = kw->val.d[0];
                  mdb[nmdb].val.d[1] = kw->val.d[1];
    }
    if (kw->pcom) {                       /* copy keyword comment    */
       pc = kw->pcom; mdb[nmdb].pcom = ps;
       while (*ps++ = *pc++);
     }
    nmdb++;
    i = 0;
  }
  else i = 1;

  return i;
}

mdb_get(mfd)
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE    get values from MIDAS descriptor buffer and store them
            in file
.RETURN     status  SC error code
---------------------------------------------------------------------*/
int    mfd;               /* IN: MIDAS file descriptor               */
{
  int   unit[4];
  int   i, n, err;
  int   ec, ed, el, ln0, ln1;

  float f;

  (void)SCECNT("GET",&ec,&el,&ed);		/* disable SC-error handling */
  ln0 = 0;  ln1 = 1;
  (void)SCECNT("PUT",&ln1,&ln0,&ln0);

  err = 0;
  for (i=0; i<nmdb; i++) {                   /* write MD-buffer      */
    switch (mdb[i].type) {
       case 'S' : err = SCDWRC(mfd,mdb[i].desc,1,mdb[i].val.pc,
			       mdb[i].idx,strlen(mdb[i].val.pc),unit);
                  break;
       case 'I' : err = SCDWRI(mfd,mdb[i].desc,&mdb[i].val.i,
			       mdb[i].idx,1,unit);
                  break;
       case 'R' : f = mdb[i].val.d[0];
                  err = SCDWRR(mfd,mdb[i].desc,&f,mdb[i].idx,1,unit);
                  break;
       case 'D' : err = SCDWRD(mfd,mdb[i].desc,&mdb[i].val.d[0],
			       mdb[i].idx,1,unit);
                  break;
    }
    if (mdb[i].pcom && 0<mdb[i].idx) 
       {
       n = (int) strlen(mdb[i].pcom);
       if (0<n) SCDWRH(mfd,mdb[i].desc,mdb[i].pcom,-1,n);
       }
  }
  nmdb = 0;

  (void)SCECNT("PUT",&ec,&el,&ed);		/* reset SC-error variables */
  return err;
}



